==> MeatAxe-2.4.8.tar.gz: mtx/2.4/src/fpcore.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: fpcore.c,v 1.1.1.1 2000/01/07 16:42:26 mringe Exp $
   Comment:     Basic factored polynomial functions.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO

#define FP_MAGIC 0x17B69244




/*{{{ --------------------------------------------------------------------------
!section obj.poly
!function FpIsValid "Check a factored polynomial"
!synopsis 
!arg p
!return
    1 if |p| points to a valid polynomial, 0 otherwise.
!description
    This function checks if the argument, |p|, is a pointer to a valid
    factored polynomial. If the polynomial is o.k., the function returns 1. 
    Otherwise, an error is signaled and, if the error handler does not 
    terminate the program, the function returns 0.
!seealso Poly_t
-------------------------------------------------------------------------- }}}*/

int FpIsValid(const FPoly_t *p)

{
    int i;
    if (p == NULL)
    {
	MTX_ERROR("NULL polynomial");
	return 0;
    }
    if (p->Magic != FP_MAGIC || p->NFactors < 0 || p->BufSize < p->NFactors)
    {
	MTX_ERROR3("Invalid FPoly_t: Magic=%d, NFactors=%d, MaxLen=%d",
	    (int)p->Magic,p->NFactors,p->BufSize);
	return 0;
    }
    if (p->Factor == NULL || p->Mult == NULL)	
    {
	MTX_ERROR2("Invalid FPoly_t: Factor:%s, Mult:%s",
	    p->Factor == 0 ? "NULL":"ok",
	    p->Mult == 0 ? "NULL":"ok");
	return 0;
    }
    for (i = 0; i < p->NFactors; ++i)
    {
	if (!PolIsValid(p->Factor[i]))
	{
	    MTX_ERROR("Invalid factor");
	    return 0;
	}
	if (p->Mult[i] < 0)
	{
	    MTX_ERROR1("Invalid multiplicity %d",p->Mult[i]);
	    return 0;
	}
	if (i > 0 && p->Factor[i]->Field != p->Factor[0]->Field)
	{
	    MTX_ERROR("Factors over different fields");
	    return 0;
	}
    }
    return 1;
}



/*{{{ --------------------------------------------------------------------------
!function FpAlloc "Allocate a factored polynomial"
!synopsis 
    FPoly_t *FpAlloc();
!return
    Pointer to the new |FPoly_t| structure, or |NULL| on error.
!description
    This function creates a new factored polynomial (|Fpoly_t|) data 
    structure. The new polynomial is empty, i.e., it has no factors.
!seealso FPoly_t FpFree
-------------------------------------------------------------------------- }}}*/

FPoly_t *FpAlloc()

{
    FPoly_t *x;

    x = ALLOC(FPoly_t);
    if (x == NULL)
    {
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->BufSize = 5;
    x->Factor = NALLOC(Poly_t *,x->BufSize);
    if (x->Factor == NULL)
    {
	SysFree(x);
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->Mult = NALLOC(int,x->BufSize);
    if (x->Mult == NULL)
    {
	SysFree(x->Factor);
	SysFree(x);
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->NFactors = 0;
    x->Magic = FP_MAGIC;
    return x;
}




/*{{{ --------------------------------------------------------------------------
!function FpFree "Free a factored polynomial"
!synopsis 
    int FpFree(FPoly_t *x);
!arg x
    Pointer to the factored polynomial.
!return
    $0$ on success, $-1$ on error.
!description
    This function frees a factored polynomial data structure, releasing
    any internally allocated memory.
!seealso FPoly_t FpAlloc
-------------------------------------------------------------------------- }}}*/

int FpFree(FPoly_t *x)

{
    int i;

    /* Check the argument
       ------------------ */
    if (!FpIsValid(x))
	return -1;

    /* Free all factors
       ---------------- */
    for (i = 0; i < x->NFactors; ++i)
	PolFree(x->Factor[i]);

    /* Free the <FPoly_t> structure
       ---------------------------- */
    SysFree(x->Factor);
    SysFree(x->Mult);
    memset(x,0,sizeof(FPoly_t));
    SysFree(x);
    return 0;
}

==> MeatAxe-2.4.9.tar.gz: mtx/2.4/src/fpcore.c <==
/* ============================= C MeatAxe ==================================
   File:        $Id: fpcore.c,v 1.1.1.1 2007/09/02 11:06:17 mringe Exp $
   Comment:     Basic factored polynomial functions.
   --------------------------------------------------------------------------
   (C) Copyright 1998 Michael Ringe, Lehrstuhl D fuer Mathematik,
   RWTH Aachen, Germany  <mringe@math.rwth-aachen.de>
   This program is free software; see the file COPYING for details.
   ========================================================================== */

#include "meataxe.h"
#include <string.h>

   
/* --------------------------------------------------------------------------
   Local data
   -------------------------------------------------------------------------- */

MTX_DEFINE_FILE_INFO

#define FP_MAGIC 0x17B69244




/**
!section obj.poly
!function FpIsValid "Check a factored polynomial"
!synopsis 
 ** @param p
 ** @return
    1 if |p| points to a valid polynomial, 0 otherwise.
!description
    This function checks if the argument, |p|, is a pointer to a valid
    factored polynomial. If the polynomial is o.k., the function returns 1. 
    Otherwise, an error is signaled and, if the error handler does not 
    terminate the program, the function returns 0.
 ** @see Poly_t
 **/

int FpIsValid(const FPoly_t *p)

{
    int i;
    if (p == NULL)
    {
	MTX_ERROR("NULL polynomial");
	return 0;
    }
    if (p->Magic != FP_MAGIC || p->NFactors < 0 || p->BufSize < p->NFactors)
    {
	MTX_ERROR3("Invalid FPoly_t: Magic=%d, NFactors=%d, MaxLen=%d",
	    (int)p->Magic,p->NFactors,p->BufSize);
	return 0;
    }
    if (p->Factor == NULL || p->Mult == NULL)	
    {
	MTX_ERROR2("Invalid FPoly_t: Factor:%s, Mult:%s",
	    p->Factor == 0 ? "NULL":"ok",
	    p->Mult == 0 ? "NULL":"ok");
	return 0;
    }
    for (i = 0; i < p->NFactors; ++i)
    {
	if (!PolIsValid(p->Factor[i]))
	{
	    MTX_ERROR("Invalid factor");
	    return 0;
	}
	if (p->Mult[i] < 0)
	{
	    MTX_ERROR1("Invalid multiplicity %d",p->Mult[i]);
	    return 0;
	}
	if (i > 0 && p->Factor[i]->Field != p->Factor[0]->Field)
	{
	    MTX_ERROR("Factors over different fields");
	    return 0;
	}
    }
    return 1;
}



/**
!function FpAlloc "Allocate a factored polynomial"
!synopsis 
    FPoly_t *FpAlloc();
 ** @return
    Pointer to the new |FPoly_t| structure, or |NULL| on error.
!description
    This function creates a new factored polynomial (|Fpoly_t|) data 
    structure. The new polynomial is empty, i.e., it has no factors.
 ** @see FPoly_t FpFree
 **/

FPoly_t *FpAlloc()

{
    FPoly_t *x;

    x = ALLOC(FPoly_t);
    if (x == NULL)
    {
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->BufSize = 5;
    x->Factor = NALLOC(Poly_t *,x->BufSize);
    if (x->Factor == NULL)
    {
	SysFree(x);
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->Mult = NALLOC(int,x->BufSize);
    if (x->Mult == NULL)
    {
	SysFree(x->Factor);
	SysFree(x);
	MTX_ERROR1("%E",MTX_ERR_NOMEM);
	return NULL;
    }
    x->NFactors = 0;
    x->Magic = FP_MAGIC;
    return x;
}




/**
!function FpFree "Free a factored polynomial"
!synopsis 
    int FpFree(FPoly_t *x);
 ** @param x
    Pointer to the factored polynomial.
 ** @return
    $0$ on success, $-1$ on error.
!description
    This function frees a factored polynomial data structure, releasing
    any internally allocated memory.
 ** @see FPoly_t FpAlloc
 **/

int FpFree(FPoly_t *x)

{
    int i;

    /* Check the argument
       ------------------ */
    if (!FpIsValid(x))
	return -1;

    /* Free all factors
       ---------------- */
    for (i = 0; i < x->NFactors; ++i)
	PolFree(x->Factor[i]);

    /* Free the <FPoly_t> structure
       ---------------------------- */
    SysFree(x->Factor);
    SysFree(x->Mult);
    memset(x,0,sizeof(FPoly_t));
    SysFree(x);
    return 0;
}
